//Stepper Motor Control
//Jake Ware and Jarvis Schultz
//IMPORTANT: You need to have the controlP5 library for processing in order to use this code

//Includes
import processing.serial.*;
import controlP5.*;

//Objects
ControlP5 controlP5;
CheckBox CheckBox1;
CheckBox CheckBox2;
Textfield TextField1;
Slider Slider1;
Serial myPort;

//Global Variables
int direction = 0;         //flag for speed control (not used in this configuration)
int positioncontrol = 0;   //flag for position control
int speed = 0;             //stores speed read from user
int requestedposition = 0; //stores position command from user
int actualposition = 0;    //current motor position that is read from serial
int dir_multiplier = 1;    //changes sign of direction when motor speed sign is changed
int zeroposition = 0;
int totalposition = 0;
int totalrequestedposition = 0;
int desiredsteps = 0;     //requested position converted to steps
int actualsteps = 0;      //steps read from motor
int state = 0;             //on or off and responsible for when to send data
int send = 0;              //send flag
String temp_desiredsteps = "00000";    //string to hold steps
String temp_speed = "000";      //string to hold speed
String temp_direction = "0";    //string to hold direction
char[] output = {'d','0','0','0','0','0','0','0','0','0'};  //holds all the data being sent to pic


//********************************************Setup**************************************
void setup () {
  frameRate(60);
  size(400, 175);            //screen size
  println(Serial.list());    //print list of available serial ports
  myPort = new Serial(this, Serial.list()[0], 115200);  //CHANGE THE [*] to select the port that you need to do your serial comm. on
  myPort.bufferUntil('\n');  //read until you see a new line
  
  //ControlP5 Setup
  //The ControlP5 website has all relevant details on these objects
  controlP5 = new ControlP5(this);
  //CreateSlider1 for speed control
  Slider1 = controlP5.addSlider("slider1",0,100,0,25,50,100,20);
  //Create TextField1 for entering desired position
  TextField1 = controlP5.addTextfield("textfield1",25,100,100,20);
  //Create CheckBox1 for direction setting
  CheckBox1 = controlP5.addCheckBox("checkbox1",25,35);  
  CheckBox1.setColorForeground(color(0,0,100));
  CheckBox1.setColorActive(color(255));
  CheckBox1.setColorLabel(color(255));
  CheckBox1.setItemsPerRow(1);
  CheckBox1.setSpacingColumn(0);
  CheckBox1.setSpacingRow(40);
  CheckBox1.addItem("forward/reverse",1);
  //Create CheckBox2 for position control enable/disable
  CheckBox2 = controlP5.addCheckBox("checkbox2",25,85);  
  CheckBox2.setColorForeground(color(0,0,100));
  CheckBox2.setColorActive(color(255));
  CheckBox2.setColorLabel(color(255));
  CheckBox2.setItemsPerRow(1);
  CheckBox2.setSpacingColumn(0);
  CheckBox2.setSpacingRow(40);
  CheckBox2.addItem("position control",2);
  //Create buttons for sending data to start or stop
  controlP5.addButton("start_button",1,5,150,80,19);
  controlP5.addButton("stop_button",1,90,150,80,19);
}


//*****************************************Draw***************************************
void draw () {
  //Create environment
  strokeWeight(0);
  background(0,0,50);
  
  //Control panel area
  fill(150);
  stroke(0);
  rect(0,0,175,height);
  
  //Motor body
  fill(100);
  ellipse(287,87,150,150);
  
  //Indicator for current motor position being sent over serial
  stroke(255,0,0);
  strokeWeight(4);
  line(287,87,287+75*cos(radians(zeroposition+actualposition)),87-75*sin(radians(zeroposition+actualposition)));
  
  //Indicator for desired motor position to be sent over serial
  if(positioncontrol == 1) {
    stroke(0,0,255);
    line(287,87,287+75*cos(radians((totalposition+dir_multiplier*requestedposition))),87-75*sin(radians((totalposition+dir_multiplier*requestedposition))));
    strokeWeight(0);
  }
  
  //set direction in output array
  temp_direction = Integer.toString(direction);
  output[1] = temp_direction.charAt(0);
  
  //set speed in output array
  temp_speed = Integer.toString(5 + 6*speed);  //scale and offset speed
  //make temp_speed string the correct length
  while(temp_speed.length() < 3) {
    temp_speed = "0" + temp_speed;
  }
  //two cases for speed greater than and equal to zero that put values into output array
  if(speed > 0) {
    for(int i = 0;i < 3;i++) {
      output[i+2] = temp_speed.charAt(i);
    }
  } else {
    for(int i = 0;i < 3;i++) {
      output[i+2] = '0';
    }
  }
  
  // Conversion for small motor:
  //desiredsteps = requestedposition*266/100;  //convert degrees to steps
  // Conversion for big motor:
  desiredsteps = requestedposition*100/340;  //convert degrees to steps
  temp_desiredsteps = Integer.toString(desiredsteps);
  //make temp_steps string the correct length
  while(temp_desiredsteps.length() < 5) {
    temp_desiredsteps = "0" + temp_desiredsteps;
  }
  //check to see if there are too many steps and put into output array
  if(temp_desiredsteps.length() < 6) {
    for(int i = 0;i < 5;i++) {
      output[i+5] = temp_desiredsteps.charAt(i);
    }
  }
  
  //send data if the send flag has been tripped
  if(send == 1) {
    if(state == 0) {
      output[0] = 'd';  //stop
    } else if(positioncontrol == 0) {
      output[0] = 's';  //motor control with just speed
    } else if(positioncontrol == 1) {
      output[0] = 'p';  //motor control with speed and position
    }
    //create output string from char array
    String temp_output = new String(output);
    println(temp_output);
    myPort.write(temp_output);
    send = 0;
  }
}


//*******************************Function Definitions*****************************
public void serialEvent (Serial myPort) {
  // get the ASCII string:
  String inString = myPort.readStringUntil('\n');

  if (inString != null) {
    inString = trim(inString);  // trim off any whitespace:
    actualsteps = int(inString);
    actualposition = -actualsteps*340/100;
  }
}

public void controlEvent(ControlEvent theEvent) {
  if(theEvent.isGroup()) {
    if(theEvent.group().name() == "checkbox1") {
      if(direction == 0) {
        direction = 1;
        dir_multiplier = -1;
      } else {
        direction = 0;
        dir_multiplier = 1;
      }
    }
    if(theEvent.group().name() == "checkbox2") {
      if(positioncontrol == 0) {
        positioncontrol = 1;
      } else {
        positioncontrol = 0;
      }
    }
    //print(direction);
    //print(" ");
    //println(positioncontrol);
  } 
}

public void textfield1(String theText) {
  requestedposition = abs(Integer.parseInt(theText));
  //println(requestedposition);
}

public void slider1(int value) {
  speed = value;
  //println(speed);
}

public void start_button(int value) {
  println("Start!");
  if(state == 0) {
    state += value;
    send = 1;
    zeroposition = totalposition;
  }
}

public void stop_button(int value) {
  println("Stop!");
  if(state == 1) {
    state -= value;
    send = 1;
    totalposition -= actualsteps*340/100;
  }
}


